@@ -88,6 +88,7 @@ public:
 private:
     inline void WritePacketID(intmax_t);
     inline void WriteTcpTxHdr(struct ReportStruct *, int, int);
+    inline void WriteTcpTxBBHdr(struct ReportStruct *, int);
     inline double get_delay_target(void);
     void InitTrafficLoop(void);
     void SetReportStartTime(void);
@@ -98,7 +99,6 @@ private:
     void PostNullEvent(void);
     void AwaitServerCloseEvent(void);
     inline void tcp_shutdown(void);
-    inline void tcp_drain(void);
     bool connected;
     ReportStruct scratchpad;
     ReportStruct *reportstruct;
@@ -139,9 +139,7 @@ private:
     Isochronous::FrameCounter *framecounter;
     bool isburst;
     bool peerclose;
-    Timestamp drain_start;
-    Timestamp drain_end;
-    struct tcp_init_conditions my_init_cond;
+    Timestamp write_start;
 }; // end class Client
 
 #endif // CLIENT_H
