@@ -71,7 +71,7 @@ public:
     // accepts connection and receives data
     void RunUDP(void);
     void RunTCP(void);
-    void RunTcpBounceBack(void);
+    void RunBounceBackTCP(void);
     static void Sig_Int(int inSigno);
 
 private:
@@ -85,8 +85,9 @@ private:
     bool InitTrafficLoop(void);
     inline void SetFullDuplexReportStartTime(void);
     inline void SetReportStartTime();
+    bool ReadBBWithRXTimestamp ();
     int ReadWithRxTimestamp(void);
-    bool ReadPacketID(void);
+    bool ReadPacketID(int);
     void L2_processing(void);
     int L2_quintuple_filter(void);
     void udp_isoch_processing(int);
