@@ -49,6 +49,7 @@
 #define PACKETRINGC_H
 
 #include "Condition.h"
+#include "gettcpinfo.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -56,14 +57,6 @@ extern "C" {
 
 #define ACKRING_DEFAULTSIZE 100
 
-struct reportstruct_tcpstats {
-    bool isValid;
-    int cwnd;
-    int rtt;
-    int rttvar;
-    intmax_t retry_tot;
-};
-
 struct ReportStruct {
     intmax_t packetID;
     intmax_t packetLen;
@@ -84,9 +77,10 @@ struct ReportStruct {
     intmax_t remaining;
     int transit_ready;
     int writecnt;
-    struct reportstruct_tcpstats tcpstats;
-    double select_delay;
-    long drain_time;
+    long write_time;
+    struct timeval sentTimeRX;
+    struct timeval sentTimeTX;
+    struct iperf_tcpstats tcpstats;
 };
 
 struct PacketRing {
