@@ -87,6 +87,11 @@ extern "C" {
 #define RUN_NOW              0x00000001
 #define HEADER16_SMALL_TRIPTIMES 0x0002 // use is 16 bits and not 32 bits
 
+// Bounceback flag
+#define HEADER_BBQUICKACK    0x8000
+#define HEADER_BBCLOCKSYNCED 0x4000 // used in the bb header only
+#define HEADER_BBTOS         0x2000
+
 // newer flags available per HEADER_EXTEND
 // Below flags are used to pass test settings in *every* UDP packet
 // and not just during the header exchange
@@ -105,11 +110,11 @@ extern "C" {
 #define HEADER_EPOCH_START    0x1000
 #define HEADER_PERIODICBURST  0x2000
 #define HEADER_WRITEPREFETCH  0x4000
+#define HEADER_TCPQUICKACK    0x8000
 
 // later features
 #define HDRXACKMAX 2500000 // default 2.5 seconds, units microseconds
-#define HDRXACKMIN   10000 // default 10 ms, units microseconds
-
+#define HDRXACKMIN   10000 // default 10 ms, units microsecond
 /*
  * Structures used for test messages which
  * are exchanged between the client and the Server/Listener
@@ -195,12 +200,9 @@ struct client_hdr_v1 {
 //    o) burst id is a running seq no
 //    o) send is the write timestamp
 //    o) bb_r2w_hold is an optional delay value between the read and bb write, typically expected as zero
-//    o) drain is the time from write to os indicating remote received payload (TCP only)
-//    o) drain times are deltas and not timestamps, computed by each end, units is microseconds
 //    o) triptimes will support OWD measurements in each direction (useful for asymmetry testing)
 //    o) min payload
 //         - seven 32b or 28 bytes with round trip only,
-//	   - nine 32b or 36 bytes when including drain times,
 //	   - eleven 32b or 44 bytes when including trip-times support
 //    o) no need for a bb read timestamp to be passed in the payload
 //    o) OWD calculations require e2e clock sync and --trip-times cli option
@@ -213,19 +215,18 @@ struct bb_ts {
     uint32_t sec;
     uint32_t usec;
 };
-union bb_r2w_info {
-    struct bb_ts bb_r2w_hold;
-    struct bb_ts bb_send;
-};
-struct bounce_back_datagram_hdr {
+struct bounceback_hdr {
     uint32_t flags;
-    uint32_t burst_size;
-    uint32_t burst_id;
-    struct bb_ts send_ts;
-    union bb_r2w_info bb_r2w; // up to here is mandatory
-    uint32_t drain; //units of usecs
-    uint32_t bb_drain;
-    struct bb_ts bb_read;
+    uint32_t bbsize;
+    uint32_t bbid;
+    uint16_t bbflags;
+    uint16_t tos;
+    struct bb_ts bbclientTx_ts;
+    struct bb_ts bbserverRx_ts;
+    struct bb_ts bbserverTx_ts;
+    uint32_t bbhold; // up to here is mandatory
+    uint32_t bbrtt;
+    struct bb_ts bbread_ts;
 };
 
 struct client_hdrext_isoch_settings {
@@ -609,7 +610,7 @@ struct server_hdr {
 #define SIZEOF_UDPHDRMSG_EXT (sizeof(struct client_udp_testhdr))
 #define SIZEOF_TCPHDRMSG_V1 (sizeof(struct client_hdr_v1))
 #define SIZEOF_TCPHDRMSG_EXT (sizeof(struct client_tcp_testhdr))
-#define MINMBUFALLOCSIZE (int) (sizeof(struct client_tcp_testhdr))
+#define MINMBUFALLOCSIZE (int) (sizeof(struct client_tcp_testhdr)) + TAPBYTESSLOP
 #define MINTRIPTIMEPLAYOAD (int) (sizeof(struct client_udp_testhdr) - sizeof(struct client_hdrext_isoch_settings))
 #ifdef __cplusplus
 } /* end extern "C" */
