@@ -97,7 +97,6 @@ Client::Client (thread_Settings *inSettings) {
 	unsetPeerVerDetect(mSettings);
     }
 
-    pattern(mSettings->mBuf, mSettings->mBufLen);
     if (isFileInput(mSettings)) {
         if (!isSTDIN(mSettings))
             Extractor_Initialize(mSettings->mFileName, mSettings->mBufLen, mSettings);
@@ -112,7 +111,7 @@ Client::Client (thread_Settings *inSettings) {
 	FAIL_errno(!(mSettings->mFPS > 0.0), "Invalid value for frames per second in the isochronous settings\n", mSettings);
     }
     peerclose = false;
-    isburst = (isIsochronous(mSettings) || isPeriodicBurst(mSettings) || ((isTripTime(mSettings) || isTcpDrain(mSettings)) && !isUDP(mSettings)));
+    isburst = (isIsochronous(mSettings) || isPeriodicBurst(mSettings) || (isTripTime(mSettings) && !isUDP(mSettings)));
 } // end Client
 
 /* -------------------------------------------------------------------
@@ -159,9 +158,16 @@ bool Client::my_connect (bool close_on_fail) {
         WARN_errno(rc == SOCKET_ERROR, "bind");
     }
 
+    if (!isUDP(mSettings) && isReport(mSettings) && isSettingsReport(mSettings)) {
+	struct ReportHeader *tmp = InitSettingsReport(mSettings);
+	assert(tmp!=NULL);
+	PostReport(tmp);
+	setNoSettReport(mSettings);
+    }
+
     // connect socket
     connected = false;
-    my_init_cond.connecttime = -1;
+    mSettings->tcpinitstats.connecttime = -1;
     if (!isUDP(mSettings)) {
 	int trycnt = mSettings->mConnectRetries + 1;
 	while (trycnt > 0) {
@@ -180,8 +186,7 @@ bool Client::my_connect (bool close_on_fail) {
 		}
 	    } else {
 		connect_done.setnow();
-		my_init_cond.connecttime = 1e3 * connect_done.subSec(connect_start);
-		mSettings->connecttime = my_init_cond.connecttime;
+		mSettings->tcpinitstats.connecttime = 1e3 * connect_done.subSec(connect_start);
 		connected = true;
 		break;
 	    }
@@ -189,20 +194,16 @@ bool Client::my_connect (bool close_on_fail) {
     } else {
 	rc = connect(mySocket, reinterpret_cast<sockaddr*>(&mSettings->peer),
 		     SockAddr_get_sizeof_sockaddr(&mSettings->peer));
-	my_init_cond.connecttime = 0.0; // UDP doesn't have a 3WHS
+	mSettings->tcpinitstats.connecttime = 0.0; // UDP doesn't have a 3WHS
         WARN_errno((rc == SOCKET_ERROR), "udp connect");
 	if (rc != SOCKET_ERROR)
 	    connected = true;
     }
-    my_init_cond.rtt = -1;
-    my_init_cond.cwnd = -1;
     if (connected) {
 #if HAVE_TCP_STATS
         assert(reportstruct);
-	if (!isUDP(mSettings) && connected) {
-	    gettcpinfo(mySocket, reportstruct);
-	    my_init_cond.rtt = reportstruct->tcpstats.rtt;
-	    my_init_cond.cwnd = reportstruct->tcpstats.cwnd;
+	if (!isUDP(mSettings)) {
+	    gettcpinfo(mySocket, &mSettings->tcpinitstats);
 	}
 #endif
 	// Set the send timeout for the very first write which has the test exchange
@@ -211,9 +212,20 @@ bool Client::my_connect (bool close_on_fail) {
 	getsockname(mySocket, reinterpret_cast<sockaddr*>(&mSettings->local), &mSettings->size_local);
 	getpeername(mySocket, reinterpret_cast<sockaddr*>(&mSettings->peer), &mSettings->size_peer);
 	SockAddr_Ifrname(mSettings);
+#ifdef DEFAULT_PAYLOAD_LEN_PER_MTU_DISCOVERY
+	if (isUDP(mSettings) && !isBuflenSet(mSettings)) {
+	    checksock_max_udp_payload(mSettings);
+	}
+#endif
 	if (isUDP(mSettings) && !isIsochronous(mSettings) && !isIPG(mSettings)) {
 	    mSettings->mBurstIPG = get_delay_target() / 1e3; // this is being set for the settings report only
 	}
+	if (isReport(mSettings) && isSettingsReport(mSettings)) {
+	    struct ReportHeader *tmp = InitSettingsReport(mSettings);
+	    assert(tmp!=NULL);
+	    PostReport(tmp);
+	    setNoSettReport(mSettings);
+	}
     } else {
 	if (mySocket != INVALID_SOCKET) {
 	    int rc = close(mySocket);
@@ -221,23 +233,17 @@ bool Client::my_connect (bool close_on_fail) {
 	    mySocket = INVALID_SOCKET;
 	}
     }
-    if (isReport(mSettings) && isSettingsReport(mSettings)) {
-	struct ReportHeader *tmp = InitSettingsReport(mSettings);
-	assert(tmp!=NULL);
-	PostReport(tmp);
-	setNoSettReport(mSettings);
-    }
     // Post the connect report unless peer version exchange is set
     if (isConnectionReport(mSettings) && !isSumOnly(mSettings)) {
 	if (connected) {
-	    struct ReportHeader *reporthdr = InitConnectionReport(mSettings, &my_init_cond);
+	    struct ReportHeader *reporthdr = InitConnectionReport(mSettings);
 	    struct ConnectionInfo *cr = static_cast<struct ConnectionInfo *>(reporthdr->this_report);
 	    cr->connect_timestamp.tv_sec = connect_start.getSecs();
 	    cr->connect_timestamp.tv_usec = connect_start.getUsecs();
 	    assert(reporthdr);
 	    PostReport(reporthdr);
 	} else {
-	    PostReport(InitConnectionReport(mSettings, &my_init_cond));
+	    PostReport(InitConnectionReport(mSettings));
 	}
     }
     return connected;
@@ -282,7 +288,7 @@ int Client::StartSynch () {
     // check for an epoch based start time
     reportstruct->packetLen = 0;
     if (!isServerReverse(mSettings)) {
-	if (!isCompat(mSettings)) {
+	if (!isCompat(mSettings) && !isBounceBack(mSettings)) {
 	    reportstruct->packetLen = SendFirstPayload();
 	    // Reverse UDP tests need to retry "first sends" a few times
 	    // before going to server or read mode
@@ -331,11 +337,11 @@ int Client::StartSynch () {
 	// Near congestion and peridiodic need sampling on every report packet
 	if (isNearCongest(mSettings) || isPeriodicBurst(mSettings)) {
 	    myReport->info.isEnableTcpInfo = true;
-	    myReport->info.ts.nextTCPStampleTime.tv_sec = 0;
-	    myReport->info.ts.nextTCPStampleTime.tv_usec = 0;
-	} else if (isEnhanced(mSettings)) {
+	    myReport->info.ts.nextTCPSampleTime.tv_sec = 0;
+	    myReport->info.ts.nextTCPSampleTime.tv_usec = 0;
+	} else if (isEnhanced(mSettings) || isBounceBack(mSettings)) {
 	    myReport->info.isEnableTcpInfo = true;
-	    myReport->info.ts.nextTCPStampleTime = myReport->info.ts.nextTime;
+	    myReport->info.ts.nextTCPSampleTime = myReport->info.ts.nextTime;
 	}
     }
 #endif
@@ -480,6 +486,7 @@ void Client::InitTrafficLoop () {
     if (isModeTime(mSettings)) {
         mEndTime.setnow();
         mEndTime.add(mSettings->mAmount / 100.0);
+	// now.setnow(); fprintf(stderr, "DEBUG: end time set to %ld.%ld now is %ld.%ld\n", mEndTime.getSecs(), mEndTime.getUsecs(), now.getSecs(), now.getUsecs());
     }
     readAt = mSettings->mBuf;
     lastPacketTime.set(myReport->info.ts.startTime.tv_sec, myReport->info.ts.startTime.tv_usec);
@@ -530,7 +537,9 @@ void Client::Run () {
 	}
     } else {
 	// Launch the approprate TCP traffic loop
-	if (mSettings->mAppRate > 0) {
+	if (isBounceBack(mSettings)) {
+	    RunBounceBackTCP();
+	} else if (mSettings->mAppRate > 0) {
 	    RunRateLimitedTCP();
 	} else if (isNearCongest(mSettings)) {
 	    RunNearCongestionTCP();
@@ -556,6 +565,7 @@ void Client::RunTCP () {
     now.setnow();
     reportstruct->packetTime.tv_sec = now.getSecs();
     reportstruct->packetTime.tv_usec = now.getUsecs();
+    reportstruct->write_time = 0;
     while (InProgress()) {
 	reportstruct->writecnt = 0;
         if (isModeAmount(mSettings)) {
@@ -580,6 +590,7 @@ void Client::RunTCP () {
 		if (isPeriodicBurst(mSettings)) {
 		    // low duty cycle traffic needs special event handling
 		    now.setnow();
+		    myReport->info.ts.prevsendTime = reportstruct->packetTime;
 		    reportstruct->packetTime.tv_sec = now.getSecs();
 		    reportstruct->packetTime.tv_usec = now.getUsecs();
 		    if (!InProgress()) {
@@ -607,10 +618,6 @@ void Client::RunTCP () {
 	    myReport->info.ts.prevsendTime = reportstruct->packetTime;
 	    writelen = (mSettings->mBufLen > burst_remaining) ? burst_remaining : mSettings->mBufLen;
 	    // perform write, full header must succeed
-#if HAVE_DECL_TCP_NOTSENT_LOWAT
-	    if (isTcpDrain(mSettings))
-		drain_start.setnow();
-#endif
 	    reportstruct->packetLen = writen(mySocket, mSettings->mBuf, writelen, &reportstruct->writecnt);
 	    FAIL_errno(reportstruct->packetLen < (intmax_t) sizeof(struct TCP_burst_payload), "burst written", mSettings);
 	} else {
@@ -618,6 +625,9 @@ void Client::RunTCP () {
 	    // perform write
 	    if (isburst)
 		writelen = (mSettings->mBufLen > burst_remaining) ? burst_remaining : mSettings->mBufLen;
+	    if (isTcpWriteTimes(mSettings)) {
+		write_start.setnow();
+	    }
 #if HAVE_DECL_TCP_NOTSENT_LOWAT
 	    if (isWritePrefetch(mSettings)) {
 		AwaitWriteSelectEventTCP();
@@ -628,6 +638,9 @@ void Client::RunTCP () {
 	    reportstruct->writecnt++;
 	    reportstruct->packetTime.tv_sec = now.getSecs();
 	    reportstruct->packetTime.tv_usec = now.getUsecs();
+	    if (isTcpWriteTimes(mSettings)) {
+		reportstruct->write_time = now.subUsec(write_start);
+	    }
 	    reportstruct->sentTime = reportstruct->packetTime;
 	}
 	if (reportstruct->packetLen <= 0) {
@@ -654,11 +667,7 @@ void Client::RunTCP () {
 		    reportstruct->transit_ready = 0;
 		} else {
 		    reportstruct->transit_ready = 1;
-#if HAVE_DECL_TCP_NOTSENT_LOWAT
-		    if (isTcpDrain(mSettings)) {
-			tcp_drain();
-		    }
-#endif
+		    reportstruct->prevSentTime = myReport->info.ts.prevsendTime;
 		}
 	    }
 	}
@@ -702,10 +711,6 @@ void Client::RunNearCongestionTCP () {
 	    WriteTcpTxHdr(reportstruct, burst_remaining, burst_id++);
 	    reportstruct->sentTime = reportstruct->packetTime;
 	    myReport->info.ts.prevsendTime = reportstruct->packetTime;
-#if HAVE_DECL_TCP_NOTSENT_LOWAT
-	    if (isTcpDrain(mSettings))
-		drain_start.setnow();
-#endif
 	    // perform write
 	    int writelen = (mSettings->mBufLen > burst_remaining) ? burst_remaining : mSettings->mBufLen;
 	    reportstruct->packetLen = write(mySocket, mSettings->mBuf, writelen);
@@ -897,9 +902,7 @@ inline bool Client::AwaitWriteSelectEventTCP (void) {
 	timeout.tv_usec = 0;
     }
 
-    Timestamp t1;
     if ((rc = select(mySocket + 1, NULL, &writeset, NULL, &timeout)) <= 0) {
-        reportstruct->select_delay = -1;
 	WARN_errno((rc < 0), "select");
 #ifdef HAVE_THREAD_DEBUG
 	if (rc == 0)
@@ -907,9 +910,6 @@ inline bool Client::AwaitWriteSelectEventTCP (void) {
 #endif
 	return false;
     }
-    Timestamp t2;
-    reportstruct->select_delay = t2.subSec(t1);
-    //    printf("*****t1 = %f\n", t2.subSec(t1));
     return true;
 }
 
@@ -925,6 +925,10 @@ void Client::RunWriteEventsTCP () {
 	    writelen = ((mSettings->mAmount < static_cast<unsigned>(mSettings->mBufLen)) ? mSettings->mAmount : mSettings->mBufLen);
 	}
 	now.setnow();
+	reportstruct->write_time = 0;
+	if (isTcpWriteTimes(mSettings)) {
+	    write_start = now;
+	}
 	bool rc = AwaitWriteSelectEventTCP();
 	reportstruct->emptyreport = (rc == false) ? 1 : 0;
         if (rc) {
@@ -941,6 +945,9 @@ void Client::RunWriteEventsTCP () {
 		}
 		reportstruct->packetLen = 0;
 		reportstruct->emptyreport = 1;
+	    } else if (isTcpWriteTimes(mSettings)) {
+		Timestamp write_end;
+		reportstruct->write_time = write_end.subUsec(write_start);
 	    }
 	}
 	if (isModeAmount(mSettings) && !reportstruct->emptyreport) {
@@ -959,7 +966,82 @@ void Client::RunWriteEventsTCP () {
 }
 #endif
 void Client::RunBounceBackTCP () {
-
+    int burst_id = 0;
+    int writelen = mSettings->mBufLen;
+    memset(mSettings->mBuf, 0x5A, sizeof(struct bounceback_hdr));
+    if (mSettings->mInterval && (mSettings->mIntervalMode == kInterval_Time)) {
+	int sotimer = static_cast<int>(round(mSettings->mInterval / 2.0));
+	SetSocketOptionsReceiveTimeout(mSettings, sotimer);
+	SetSocketOptionsSendTimeout(mSettings, sotimer);
+    } else if (isModeTime(mSettings)) {
+	int sotimer = static_cast<int>(round(mSettings->mAmount * 10000) / 2);
+	SetSocketOptionsReceiveTimeout(mSettings, sotimer);
+	SetSocketOptionsSendTimeout(mSettings, sotimer);
+    }
+    if (isModeTime(mSettings)) {
+	int end_usecs  (mSettings->mAmount * 10000); //amount units is 10 ms
+	if (int err = set_itimer(end_usecs)) {
+	    FAIL_errno(err != 0, "setitimer", mSettings);
+	}
+    }
+    now.setnow();
+    reportstruct->packetTime.tv_sec = now.getSecs();
+    reportstruct->packetTime.tv_usec = now.getUsecs();
+    while (InProgress()) {
+	int n;
+	reportstruct->writecnt = 0;
+	if (framecounter) {
+	    burst_id = framecounter->wait_tick();
+	} else {
+	    burst_id++;
+	}
+	now.setnow();
+	reportstruct->sentTime.tv_sec = now.getSecs();
+	reportstruct->sentTime.tv_usec = now.getUsecs();
+	WriteTcpTxBBHdr(reportstruct, burst_id);
+	myReport->info.ts.prevsendTime = reportstruct->sentTime;
+	reportstruct->packetLen = writen(mySocket, mSettings->mBuf, writelen, &reportstruct->writecnt);
+	if (reportstruct->packetLen == writelen) {
+	    reportstruct->emptyreport = 0;
+	    totLen += reportstruct->packetLen;
+	    reportstruct->errwrite=WriteNoErr;
+#if HAVE_DECL_TCP_QUICKACK
+	    if (isTcpQuickAck(mSettings)) {
+		int opt = 1;
+		Socklen_t len = sizeof(opt);
+		int rc = setsockopt(mySocket, IPPROTO_TCP, TCP_QUICKACK,
+				    reinterpret_cast<char*>(&opt), len);
+		WARN_errno(rc == SOCKET_ERROR, "setsockopt TCP_QUICKACK");
+	    }
+#endif
+	    if ((n = recvn(mySocket, mSettings->mBuf, mSettings->mBounceBackBytes, 0)) == mSettings->mBounceBackBytes) {
+		struct bounceback_hdr *bbhdr = reinterpret_cast<struct bounceback_hdr *>(mSettings->mBuf);
+		now.setnow();
+		reportstruct->sentTimeRX.tv_sec = ntohl(bbhdr->bbserverRx_ts.sec);
+		reportstruct->sentTimeRX.tv_usec = ntohl(bbhdr->bbserverRx_ts.usec);
+		reportstruct->sentTimeTX.tv_sec = ntohl(bbhdr->bbserverTx_ts.sec);
+		reportstruct->sentTimeTX.tv_usec = ntohl(bbhdr->bbserverTx_ts.usec);
+		reportstruct->packetTime.tv_sec = now.getSecs();
+		reportstruct->packetTime.tv_usec = now.getUsecs();
+		reportstruct->packetLen += n;
+		reportstruct->emptyreport = 0;
+		myReportPacket();
+	    } else if (n == 0) {
+		peerclose = true;
+	    }
+	} else if ((reportstruct->packetLen < 0 ) && NONFATALTCPWRITERR(errno)) {
+	    reportstruct->packetLen = 0;
+	    reportstruct->emptyreport = 1;
+	    reportstruct->errwrite=WriteErrNoAccount;
+	    myReportPacket();
+	} else {
+	    reportstruct->errwrite=WriteErrFatal;
+	    reportstruct->packetLen = -1;
+	    FAIL_errno(1, "tcp bounce-back write", mSettings);
+	}
+    }
+    disarm_itimer();
+    FinishTrafficActions();
 }
 /*
  * UDP send loop
@@ -1007,6 +1089,7 @@ void Client::RunUDP () {
         //  default: break;
         //}
 	now.setnow();
+	reportstruct->writecnt = 1;
 	reportstruct->packetTime.tv_sec = now.getSecs();
 	reportstruct->packetTime.tv_usec = now.getUsecs();
 	reportstruct->sentTime = reportstruct->packetTime;
@@ -1305,6 +1388,35 @@ inline void Client::WriteTcpTxHdr (struct ReportStruct *reportstruct, int burst_
 //    printf("**** Write tcp burst header size= %d id = %d\n", burst_size, burst_id);
 }
 
+// See payloads.h
+void Client::WriteTcpTxBBHdr (struct ReportStruct *reportstruct, int bbid) {
+    struct bounceback_hdr * mBuf_bb = reinterpret_cast<struct bounceback_hdr *>(mSettings->mBuf);
+    // store packet ID into buffer
+    uint32_t flags = HEADER_BOUNCEBACK;
+    uint32_t bbflags = 0x0;
+    mBuf_bb->flags = htonl(flags);
+    if (isTripTime(mSettings)) {
+	bbflags |= HEADER_BBCLOCKSYNCED;
+    }
+    if (mSettings->mTOS) {
+	bbflags |= HEADER_BBTOS;
+	mBuf_bb->tos = htons((mSettings->mTOS & 0xFF));
+    }
+    if (isTcpQuickAck(mSettings)) {
+	bbflags |= HEADER_BBQUICKACK;
+    }
+    mBuf_bb->bbflags = htons(bbflags);
+    mBuf_bb->bbsize = htonl(mSettings->mBufLen);
+    mBuf_bb->bbid = htonl(bbid);
+    mBuf_bb->bbclientTx_ts.sec = htonl(reportstruct->packetTime.tv_sec);
+    mBuf_bb->bbclientTx_ts.usec = htonl(reportstruct->packetTime.tv_usec);
+    mBuf_bb->bbserverRx_ts.sec = -1;
+    mBuf_bb->bbserverRx_ts.usec = -1;
+    mBuf_bb->bbserverTx_ts.sec = -1;
+    mBuf_bb->bbserverTx_ts.usec = -1;
+    mBuf_bb->bbhold = htonl(mSettings->mBounceBackHold);
+}
+
 inline bool Client::InProgress (void) {
     // Read the next data block from
     // the file if it's file input
@@ -1312,22 +1424,12 @@ inline bool Client::InProgress (void) {
 	Extractor_getNextDataBlock(readAt, mSettings);
         return Extractor_canRead(mSettings) != 0;
     }
+    // fprintf(stderr, "DEBUG: SI=%d PC=%d T=%d A=%d\n", sInterupted, peerclose, (isModeTime(mSettings) && mEndTime.before(reportstruct->packetTime)), (isModeAmount(mSettings) && (mSettings->mAmount <= 0)));
     return !(sInterupted || peerclose || \
 	(isModeTime(mSettings) && mEndTime.before(reportstruct->packetTime))  ||
 	(isModeAmount(mSettings) && (mSettings->mAmount <= 0)));
 }
 
-inline void Client::tcp_drain (void) {
-#if HAVE_DECL_TCP_NOTSENT_LOWAT
-    AwaitWriteSelectEventTCP();
-    drain_end.setnow();
-    reportstruct->drain_time = drain_end.subUsec(drain_start);
-#ifdef HAVE_THREAD_DEBUG
-    thread_debug("Drain time  = %f", reportstruct->drain_time);
-#endif
-#endif
-}
-
 inline void Client::tcp_shutdown (void) {
     if ((mySocket != INVALID_SOCKET) && isConnected()) {
 	int rc = shutdown(mySocket, SHUT_WR);
@@ -1526,6 +1628,7 @@ int Client::SendFirstPayload () {
 	    reportstruct->packetTime.tv_sec = now.getSecs();
 	    reportstruct->packetTime.tv_usec = now.getUsecs();
 	}
+	pattern(mSettings->mBuf, mSettings->mBufLen);
 	if (isTxStartTime(mSettings)) {
 	    pktlen = Settings_GenerateClientHdr(mSettings, (void *) mSettings->mBuf, mSettings->txstart_epoch);
 	} else {
