@@ -57,16 +57,17 @@
 #define SNBUFFEREXTENDSIZE 512
 static char outbuffer[SNBUFFERSIZE]; // Buffer for printing
 static char outbufferext[SNBUFFEREXTENDSIZE]; // Buffer for printing
-static char outbufferext2[SNBUFFEREXTENDSIZE]; // Buffer for printing
+
 static char llaw_buf[100];
 static char netpower_buf[100];
 
 static int HEADING_FLAG(report_bw) = 0;
+static int HEADING_FLAG(report_client_bb_bw) = 0;
 static int HEADING_FLAG(report_bw_jitter_loss) = 0;
 static int HEADING_FLAG(report_bw_read_enhanced) = 0;
 static int HEADING_FLAG(report_bw_read_enhanced_netpwr) = 0;
 static int HEADING_FLAG(report_bw_write_enhanced) = 0;
-static int HEADING_FLAG(report_write_enhanced_drain) = 0;
+static int HEADING_FLAG(report_write_enhanced_write) = 0;
 static int HEADING_FLAG(report_bw_write_enhanced_netpwr) = 0;
 static int HEADING_FLAG(report_bw_pps_enhanced) = 0;
 static int HEADING_FLAG(report_bw_pps_enhanced_isoch) = 0;
@@ -87,16 +88,18 @@ static int HEADING_FLAG(report_bw_jitter_loss_enhanced_triptime) = 0;
 static int HEADING_FLAG(report_bw_jitter_loss_enhanced_isoch_triptime) = 0;
 static int HEADING_FLAG(report_sumcnt_bw_jitter_loss) = 0;
 static int HEADING_FLAG(report_burst_read_tcp) = 0;
+static int HEADING_FLAG(report_burst_write_tcp) = 0;
 
 void reporter_default_heading_flags (int flag) {
     HEADING_FLAG(report_bw) = flag;
+    HEADING_FLAG(report_client_bb_bw) = flag;
     HEADING_FLAG(report_sumcnt_bw) = flag;
     HEADING_FLAG(report_sumcnt_udp_fullduplex) = flag;
     HEADING_FLAG(report_bw_jitter_loss) = flag;
     HEADING_FLAG(report_bw_read_enhanced) = flag;
     HEADING_FLAG(report_bw_read_enhanced_netpwr) = flag;
     HEADING_FLAG(report_bw_write_enhanced) = flag;
-    HEADING_FLAG(report_write_enhanced_drain) = flag;
+    HEADING_FLAG(report_write_enhanced_write) = flag;
     HEADING_FLAG(report_write_enhanced_isoch) = flag;
     HEADING_FLAG(report_bw_write_enhanced_netpwr) = flag;
     HEADING_FLAG(report_bw_pps_enhanced) = flag;
@@ -113,6 +116,7 @@ void reporter_default_heading_flags (int flag) {
     HEADING_FLAG(report_sumcnt_bw_jitter_loss) = flag;
     HEADING_FLAG(report_sumcnt_bw_pps_enhanced) = flag;
     HEADING_FLAG(report_burst_read_tcp) = flag;
+    HEADING_FLAG(report_burst_write_tcp) = flag;
 }
 static inline void _print_stats_common (struct TransferInfo *stats) {
     assert(stats!=NULL);
@@ -177,8 +181,12 @@ static inline void set_netpowerbuf(double meantransit, struct TransferInfo *stat
       double netpwr = NETPOWERCONSTANT * (((double) stats->cntBytes) / (stats->ts.iEnd - stats->ts.iStart) / meantransit);
       if (netpwr <  NETPWR_LOWERBOUNDS) {
 	  strcpy(netpower_buf, "OBL");
-      } else {
+      } else if (netpwr > 100)  {
 	  snprintf(netpower_buf, sizeof(netpower_buf), "%.0f", netpwr);
+      } else if (netpwr > 10)  {
+	  snprintf(netpower_buf, sizeof(netpower_buf), "%.2f", netpwr);
+      } else {
+	  snprintf(netpower_buf, sizeof(netpower_buf), "%.6f", netpwr);
       }
   }
 }
@@ -231,7 +239,7 @@ void tcp_output_read_enhanced (struct TransferInfo *stats) {
 }
 void tcp_output_read_enhanced_triptime (struct TransferInfo *stats) {
     HEADING_PRINT_COND(report_bw_read_enhanced_netpwr);
-    double meantransit = (stats->transit.cntTransit > 0) ? (stats->transit.sumTransit / stats->transit.cntTransit) : 0;
+    double meantransit = (stats->transit.current.cnt > 0) ? (stats->transit.current.sum / stats->transit.current.cnt) : 0;
     double lambda = (stats->IPGsum > 0.0) ? ((double)stats->cntBytes / stats->IPGsum) : 0.0;
     set_llawbuf(lambda, meantransit, stats);
     _print_stats_common(stats);
@@ -241,11 +249,11 @@ void tcp_output_read_enhanced_triptime (struct TransferInfo *stats) {
 	       stats->common->transferIDStr, stats->ts.iStart, stats->ts.iEnd,
 	       outbuffer, outbufferext,
 	       (meantransit * 1e3),
-	       (stats->transit.cntTransit < 2) ? 0 : stats->transit.minTransit*1e3,
-	       (stats->transit.cntTransit < 2) ? 0 : stats->transit.maxTransit*1e3,
-	       (stats->transit.cntTransit < 2) ? 0 : sqrt(stats->transit.m2Transit / (stats->transit.cntTransit - 1)) / 1e3,
-	       stats->transit.cntTransit,
-	       stats->transit.cntTransit ? (long) ((double)stats->cntBytes / (double) stats->transit.cntTransit) : 0,
+	       (stats->transit.current.cnt < 2) ? 0 : stats->transit.current.min * 1e3,
+	       (stats->transit.current.cnt < 2) ? 0 : stats->transit.current.max * 1e3,
+	       (stats->transit.current.cnt < 2) ? 0 : 1e3 * (sqrt(stats->transit.current.m2 / (stats->transit.current.cnt - 1))),
+	       stats->transit.current.cnt,
+	       stats->transit.current.cnt ? (long) ((double)stats->cntBytes / (double) stats->transit.current.cnt) : 0,
 	       llaw_buf,
 	       netpower_buf,
 	       stats->sock_callstats.read.cntRead,
@@ -263,11 +271,11 @@ void tcp_output_read_enhanced_triptime (struct TransferInfo *stats) {
 	       stats->common->transferIDStr, stats->ts.iStart, stats->ts.iEnd,
 	       outbuffer, outbufferext,
 	       (meantransit * 1e3),
-	       stats->transit.minTransit*1e3,
-	       stats->transit.maxTransit*1e3,
-	       (stats->transit.cntTransit < 2) ? 0 : sqrt(stats->transit.m2Transit / (stats->transit.cntTransit - 1)) / 1e3,
-	       stats->transit.cntTransit,
-	       stats->transit.cntTransit ? (long) ((double)stats->cntBytes / (double) stats->transit.cntTransit) : 0,
+	       stats->transit.current.min * 1e3,
+	       stats->transit.current.max * 1e3,
+	       (stats->transit.current.cnt < 2) ? 0 : 1e3 * (sqrt(stats->transit.current.m2 / (stats->transit.current.cnt - 1))),
+	       stats->transit.current.cnt,
+	       stats->transit.current.cnt ? (long) ((double)stats->cntBytes / (double) stats->transit.current.cnt) : 0,
 	       llaw_buf,
 	       netpower_buf,
 	       stats->sock_callstats.read.cntRead,
@@ -309,11 +317,12 @@ void tcp_output_burst_read (struct TransferInfo *stats) {
     HEADING_PRINT_COND(report_burst_read_tcp);
     _print_stats_common(stats);
     if (!stats->final) {
-	set_netpowerbuf(stats->tripTime, stats);
+	set_netpowerbuf(stats->transit.current.mean, stats);
 	printf(report_burst_read_tcp_format,
 	       stats->common->transferIDStr, stats->ts.iStart, stats->ts.iEnd,
 	       outbuffer, outbufferext,
-	       stats->tripTime,
+	       stats->transit.current.mean * 1e3,
+	       (1e2 * stats->transit.current.mean * stats->common->FPS), // (1e3 / 100%)
 	       stats->sock_callstats.read.cntRead,
 	       stats->sock_callstats.read.bins[0],
 	       stats->sock_callstats.read.bins[1],
@@ -323,12 +332,15 @@ void tcp_output_burst_read (struct TransferInfo *stats) {
 	       stats->sock_callstats.read.bins[5],
 	       stats->sock_callstats.read.bins[6],
 	       stats->sock_callstats.read.bins[7],
-	       (stats->tripTime * stats->common->FPS) / 10.0, // (1e3 / 100%)
 	       netpower_buf);
     } else {
 	printf(report_burst_read_tcp_final_format,
 	       stats->common->transferIDStr, stats->ts.iStart, stats->ts.iEnd,
 	       outbuffer, outbufferext,
+	       stats->transit.total.mean * 1e3,
+	       (stats->transit.total.cnt < 2) ? 0 : stats->transit.total.min * 1e3,
+	       (stats->transit.total.cnt < 2) ? 0 : stats->transit.total.max * 1e3,
+	       (stats->transit.total.cnt < 2) ? 0 : 1e3 * (sqrt(stats->transit.total.m2 / (stats->transit.total.cnt - 1))),
 	       stats->sock_callstats.read.cntRead,
 	       stats->sock_callstats.read.bins[0],
 	       stats->sock_callstats.read.bins[1],
@@ -352,6 +364,112 @@ void tcp_output_write (struct TransferInfo *stats) {
     fflush(stdout);
 }
 
+void tcp_output_write_bb (struct TransferInfo *stats) {
+    HEADING_PRINT_COND(report_client_bb_bw);
+    _print_stats_common(stats);
+    if (stats->final) {
+	int cnt_sec = (stats->bbrtt.total.cnt > 0) ? ((int) ((double) stats->bbrtt.total.cnt / (stats->ts.iEnd - stats->ts.iStart))) : 0;
+#if HAVE_TCP_STATS
+	printf(report_client_bb_bw_format, stats->common->transferIDStr,
+	       stats->ts.iStart, stats->ts.iEnd,
+	       outbuffer, outbufferext,
+	       stats->bbrtt.total.cnt,
+	       cnt_sec,
+	       (stats->bbrtt.total.mean * 1e3),
+	       (stats->bbrtt.total.cnt < 2) ? 0 : (stats->bbrtt.total.min * 1e3),
+	       (stats->bbrtt.total.cnt < 2) ? 0 : (stats->bbrtt.total.max * 1e3),
+	       (stats->bbrtt.total.cnt < 2) ? 0 : 1e3 * (sqrt(stats->bbrtt.total.m2 / (stats->bbrtt.total.cnt - 1))),
+	       stats->sock_callstats.write.tcpstats.retry,
+	       stats->sock_callstats.write.tcpstats.cwnd,
+	       stats->sock_callstats.write.tcpstats.rtt);
+#else
+	printf(report_client_bb_bw_format, stats->common->transferIDStr,
+	       stats->ts.iStart, stats->ts.iEnd,
+	       outbuffer, outbufferext,
+	       stats->bbrtt.total.cnt,
+	       cnt_sec,
+	       (stats->bbrtt.total.mean * 1e3),
+	       (stats->bbrtt.total.cnt < 2) ? 0 : (stats->bbrtt.total.min * 1e3),
+	       (stats->bbrtt.total.cnt < 2) ? 0 : (stats->bbrtt.total.max * 1e3),
+	       (stats->bbrtt.total.cnt < 2) ? 0 : 1e3 * (sqrt(stats->bbrtt.total.m2 / (stats->bbrtt.total.cnt - 1))));
+#endif
+	if (isTripTime(stats->common)) {
+	    printf(report_client_bb_bw_triptime_format, stats->common->transferIDStr,
+		   stats->ts.iStart, stats->ts.iEnd,
+		   stats->bbowdto.total.cnt,
+		   cnt_sec,
+		   (stats->bbowdto.total.mean * 1e3),
+		   (stats->bbowdto.total.cnt < 2) ? 0 : (stats->bbowdto.total.min * 1e3),
+		   (stats->bbowdto.total.cnt < 2) ? 0 : (stats->bbowdto.total.max * 1e3),
+		   (stats->bbowdto.total.cnt < 2) ? 0 : 1e3 * (sqrt(stats->bbowdto.total.m2 / (stats->bbowdto.total.cnt - 1))),
+		   (stats->bbowdfro.total.mean * 1e3),
+		   (stats->bbowdfro.total.cnt < 2) ? 0 : (stats->bbowdfro.total.min * 1e3),
+		   (stats->bbowdfro.total.cnt < 2) ? 0 : (stats->bbowdfro.total.max * 1e3),
+		   (stats->bbowdfro.total.cnt < 2) ? 0 : 1e3 * (sqrt(stats->bbowdfro.total.m2 / (stats->bbowdfro.total.cnt - 1))),
+		   (stats->bbasym.total.mean * 1e3),
+		   (stats->bbasym.total.cnt < 2) ? 0 : (stats->bbasym.total.min * 1e3),
+		   (stats->bbasym.total.cnt < 2) ? 0 : (stats->bbasym.total.max * 1e3),
+		   (stats->bbasym.total.cnt < 2) ? 0 : 1e3 * (sqrt(stats->bbasym.total.m2 / (stats->bbasym.total.cnt - 1))));
+	}
+	if (stats->bbrtt_histogram) {
+	  histogram_print(stats->bbrtt_histogram, stats->ts.iStart, stats->ts.iEnd);
+	}
+    } else {
+	int cnt_sec = (stats->bbrtt.total.cnt > 0) ? ((int) ((double) stats->bbrtt.current.cnt / (stats->ts.iEnd - stats->ts.iStart))) : 0;
+#if HAVE_TCP_STATS
+	printf(report_client_bb_bw_format, stats->common->transferIDStr,
+	       stats->ts.iStart, stats->ts.iEnd,
+	       outbuffer, outbufferext,
+	       stats->bbrtt.current.cnt,
+	       cnt_sec,
+	       (stats->bbrtt.current.mean * 1e3),
+	       (stats->bbrtt.current.cnt < 2) ? 0 : (stats->bbrtt.current.min * 1e3),
+	       (stats->bbrtt.current.cnt < 2) ? 0 : (stats->bbrtt.current.max * 1e3),
+	       (stats->bbrtt.current.cnt < 2) ? 0 : 1e3 * (sqrt(stats->bbrtt.current.m2 / (stats->bbrtt.current.cnt - 1))),
+	       stats->sock_callstats.write.tcpstats.retry,
+	       stats->sock_callstats.write.tcpstats.cwnd,
+	       stats->sock_callstats.write.tcpstats.rtt);
+#else
+	printf(report_client_bb_bw_format, stats->common->transferIDStr,
+	       stats->ts.iStart, stats->ts.iEnd,
+	       outbuffer, outbufferext,
+	       stats->bbrtt.current.cnt,
+	       cnt_sec,
+	       (stats->bbrtt.current.mean * 1e3),
+	       (stats->bbrtt.current.cnt < 2) ? 0 : (stats->bbrtt.current.min * 1e3),
+	       (stats->bbrtt.current.cnt < 2) ? 0 : (stats->bbrtt.current.max * 1e3),
+	       (stats->bbrtt.current.cnt < 2) ? 0 : 1e3 * (sqrt(stats->bbrtt.current.m2 / (stats->bbrtt.current.cnt - 1))));
+#endif
+    }
+    fflush(stdout);
+}
+
+void tcp_output_burst_write (struct TransferInfo *stats) {
+    HEADING_PRINT_COND(report_burst_write_tcp);
+    _print_stats_common(stats);
+#if HAVE_TCP_STATS
+    set_netpowerbuf((stats->transit.current.mean + stats->sock_callstats.write.tcpstats.rtt), stats);
+    printf(report_burst_write_tcp_format, stats->common->transferIDStr,
+	   stats->ts.iStart, stats->ts.iEnd,
+	   outbuffer, outbufferext,
+	   stats->transit.current.mean,
+	   stats->sock_callstats.write.WriteCnt,
+	   stats->sock_callstats.write.WriteErr,
+	   stats->sock_callstats.write.tcpstats.retry,
+	   stats->sock_callstats.write.tcpstats.cwnd,
+	   stats->sock_callstats.write.tcpstats.rtt,
+	   netpower_buf);
+ #else
+    printf(report_burst_write_tcp_format, stats->common->transferIDStr,
+	   stats->ts.iStart, stats->ts.iEnd,
+	   outbuffer, outbufferext,
+	   stats->transit.current.mean,
+	   stats->sock_callstats.write.WriteCnt,
+	   stats->sock_callstats.write.WriteErr);
+#endif
+    fflush(stdout);
+}
+
 void tcp_output_write_enhanced (struct TransferInfo *stats) {
     HEADING_PRINT_COND(report_bw_write_enhanced);
     _print_stats_common(stats);
@@ -362,17 +480,17 @@ void tcp_output_write_enhanced (struct TransferInfo *stats) {
 	   stats->sock_callstats.write.WriteCnt,
 	   stats->sock_callstats.write.WriteErr);
 #else
-    set_netpowerbuf(stats->sock_callstats.write.rtt * 1e-6, stats);
-    if (stats->sock_callstats.write.cwnd > 0) {
+    set_netpowerbuf(stats->sock_callstats.write.tcpstats.rtt * 1e-6, stats);
+    if (stats->sock_callstats.write.tcpstats.cwnd > 0) {
 	printf(report_bw_write_enhanced_format,
 	       stats->common->transferIDStr, stats->ts.iStart, stats->ts.iEnd,
 	       outbuffer, outbufferext,
 	       stats->sock_callstats.write.WriteCnt,
 	       stats->sock_callstats.write.WriteErr,
-	       stats->sock_callstats.write.TCPretry,
-	       stats->sock_callstats.write.cwnd,
-	       stats->sock_callstats.write.rtt,
-	       stats->sock_callstats.write.rttvar,
+	       stats->sock_callstats.write.tcpstats.retry,
+	       stats->sock_callstats.write.tcpstats.cwnd,
+	       stats->sock_callstats.write.tcpstats.rtt,
+	       stats->sock_callstats.write.tcpstats.rttvar,
 	       netpower_buf);
     } else {
 	printf(report_bw_write_enhanced_nocwnd_format,
@@ -380,8 +498,8 @@ void tcp_output_write_enhanced (struct TransferInfo *stats) {
 	       outbuffer, outbufferext,
 	       stats->sock_callstats.write.WriteCnt,
 	       stats->sock_callstats.write.WriteErr,
-	       stats->sock_callstats.write.TCPretry,
-	       stats->sock_callstats.write.rtt,
+	       stats->sock_callstats.write.tcpstats.retry,
+	       stats->sock_callstats.write.tcpstats.rtt,
 	       netpower_buf);
     }
 #endif
@@ -393,66 +511,64 @@ void tcp_output_write_enhanced (struct TransferInfo *stats) {
     fflush(stdout);
 }
 
-#if HAVE_DECL_TCP_NOTSENT_LOWAT
-void tcp_output_write_enhanced_drain (struct TransferInfo *stats) {
-    HEADING_PRINT_COND(report_write_enhanced_drain);
+void tcp_output_write_enhanced_write (struct TransferInfo *stats) {
+    HEADING_PRINT_COND(report_write_enhanced_write);
     _print_stats_common(stats);
 #if !(HAVE_TCP_STATS)
-    printf(report_write_enhanced_drain_format,
+    printf(report_write_enhanced_write_format,
 	   stats->common->transferIDStr, stats->ts.iStart, stats->ts.iEnd,
 	   outbuffer, outbufferext,
 	   stats->sock_callstats.write.WriteCnt,
 	   stats->sock_callstats.write.WriteErr,
-
-	   stats->drain_mmm.current.mean * 1e-3,
-	   stats->drain_mmm.current.min * 1e-3,
-	   stats->drain_mmm.current.max * 1e-3,
-	   (stats->drain_mmm.current.cnt < 2) ? 0 : (1e-3 * sqrt(stats->drain_mmm.current.m2 / (stats->drain_mmm.current.cnt - 1))),
-	   stats->drain_mmm.current.cnt);
+	   stats->write_mmm.current.mean * 1e3,
+	   stats->write_mmm.current.min * 1e3,
+	   stats->write_mmm.current.max * 1e3,
+	   (stats->write_mmm.current.cnt < 2) ? 0 : (1e-3 * sqrt(stats->write_mmm.current.m2 / (stats->write_mmm.current.cnt - 1))),
+	   stats->write_mmm.current.cnt);
 #else
-    set_netpowerbuf(stats->sock_callstats.write.rtt * 1e-6, stats);
-    if (stats->sock_callstats.write.cwnd > 0) {
-	printf(report_write_enhanced_drain_format,
+    set_netpowerbuf(stats->sock_callstats.write.tcpstats.rtt * 1e-6, stats);
+    if (stats->sock_callstats.write.tcpstats.cwnd > 0) {
+	printf(report_write_enhanced_write_format,
 	       stats->common->transferIDStr, stats->ts.iStart, stats->ts.iEnd,
 	       outbuffer, outbufferext,
 	       stats->sock_callstats.write.WriteCnt,
 	       stats->sock_callstats.write.WriteErr,
-	       stats->sock_callstats.write.TCPretry,
-	       stats->sock_callstats.write.cwnd,
-	       stats->sock_callstats.write.rtt,
+	       stats->sock_callstats.write.tcpstats.retry,
+	       stats->sock_callstats.write.tcpstats.cwnd,
+	       stats->sock_callstats.write.tcpstats.rtt,
 	       netpower_buf,
-	       stats->drain_mmm.current.cnt,
-	       stats->drain_mmm.current.mean * 1e-3,
-	       stats->drain_mmm.current.min * 1e-3,
-	       stats->drain_mmm.current.max * 1e-3,
-	       (stats->drain_mmm.current.cnt < 2) ? 0 : (1e-3 * sqrt(stats->drain_mmm.current.m2 / (stats->drain_mmm.current.cnt - 1))),
-	       stats->drain_mmm.current.cnt);
+	       stats->write_mmm.current.cnt,
+	       stats->write_mmm.current.mean * 1e-3,
+	       stats->write_mmm.current.min * 1e-3,
+	       stats->write_mmm.current.max * 1e-3,
+	       (stats->write_mmm.current.cnt < 2) ? 0 : (1e-3 * sqrt(stats->write_mmm.current.m2 / (stats->write_mmm.current.cnt - 1))),
+	       stats->write_mmm.current.cnt);
     } else {
-	printf(report_write_enhanced_nocwnd_drain_format,
+	printf(report_write_enhanced_nocwnd_write_format,
 	       stats->common->transferIDStr, stats->ts.iStart, stats->ts.iEnd,
 	       outbuffer, outbufferext,
 	       stats->sock_callstats.write.WriteCnt,
 	       stats->sock_callstats.write.WriteErr,
-	       stats->sock_callstats.write.TCPretry,
-	       stats->sock_callstats.write.rtt,
+	       stats->sock_callstats.write.tcpstats.retry,
+	       stats->sock_callstats.write.tcpstats.rtt,
 	       netpower_buf,
-	       stats->drain_mmm.current.cnt,
-	       stats->drain_mmm.current.mean * 1e-3,
-	       stats->drain_mmm.current.min * 1e-3,
-	       stats->drain_mmm.current.max * 1e-3,
-	       (stats->drain_mmm.current.cnt < 2) ? 0 : (1e-3 * sqrt(stats->drain_mmm.current.m2 / (stats->drain_mmm.current.cnt - 1))),
-	       stats->drain_mmm.current.cnt);
+	       stats->write_mmm.current.cnt,
+	       stats->write_mmm.current.mean * 1e3,
+	       stats->write_mmm.current.min * 1e3,
+	       stats->write_mmm.current.max * 1e3,
+	       (stats->write_mmm.current.cnt < 2) ? 0 : (1e3 * sqrt(stats->write_mmm.current.m2 / (stats->write_mmm.current.cnt - 1))),
+	       stats->write_mmm.current.cnt);
     }
 #endif
     if (stats->latency_histogram) {
 	histogram_print(stats->latency_histogram, stats->ts.iStart, stats->ts.iEnd);
     }
-    if (stats->drain_histogram) {
-	histogram_print(stats->drain_histogram, stats->ts.iStart, stats->ts.iEnd);
+    if (stats->write_histogram) {
+	histogram_print(stats->write_histogram, stats->ts.iStart, stats->ts.iEnd);
     }
     fflush(stdout);
 }
-#endif
+
 void tcp_output_write_enhanced_isoch (struct TransferInfo *stats) {
     HEADING_PRINT_COND(report_write_enhanced_isoch);
     _print_stats_common(stats);
@@ -464,16 +580,16 @@ void tcp_output_write_enhanced_isoch (struct TransferInfo *stats) {
 	   stats->sock_callstats.write.WriteErr,
 	   stats->isochstats.cntFrames, stats->isochstats.cntFramesMissed, stats->isochstats.cntSlips);
 #else
-    set_netpowerbuf(stats->sock_callstats.write.rtt * 1e-6, stats);
-    if (stats->sock_callstats.write.cwnd > 0) {
+    set_netpowerbuf(stats->sock_callstats.write.tcpstats.rtt * 1e-6, stats);
+    if (stats->sock_callstats.write.tcpstats.cwnd > 0) {
 	printf(report_write_enhanced_isoch_format,
 	       stats->common->transferIDStr, stats->ts.iStart, stats->ts.iEnd,
 	       outbuffer, outbufferext,
 	       stats->sock_callstats.write.WriteCnt,
 	       stats->sock_callstats.write.WriteErr,
-	       stats->sock_callstats.write.TCPretry,
-	       stats->sock_callstats.write.cwnd,
-	       stats->sock_callstats.write.rtt,
+	       stats->sock_callstats.write.tcpstats.retry,
+	       stats->sock_callstats.write.tcpstats.cwnd,
+	       stats->sock_callstats.write.tcpstats.rtt,
 	       stats->isochstats.cntFrames, stats->isochstats.cntFramesMissed, stats->isochstats.cntSlips,
 	       netpower_buf);
     } else {
@@ -482,8 +598,8 @@ void tcp_output_write_enhanced_isoch (struct TransferInfo *stats) {
 	       outbuffer, outbufferext,
 	       stats->sock_callstats.write.WriteCnt,
 	       stats->sock_callstats.write.WriteErr,
-	       stats->sock_callstats.write.TCPretry,
-	       stats->sock_callstats.write.rtt,
+	       stats->sock_callstats.write.tcpstats.retry,
+	       stats->sock_callstats.write.tcpstats.rtt,
 	       stats->isochstats.cntFrames, stats->isochstats.cntFramesMissed, stats->isochstats.cntSlips,
 	       netpower_buf);
     }
@@ -529,57 +645,16 @@ void udp_output_read (struct TransferInfo *stats) {
     printf(report_bw_jitter_loss_format, stats->common->transferIDStr,
 	    stats->ts.iStart, stats->ts.iEnd,
 	    outbuffer, outbufferext,
-	    stats->jitter*1000.0, stats->cntError, stats->cntDatagrams,
+	    (stats->jitter * 1e3), stats->cntError, stats->cntDatagrams,
 	    (100.0 * stats->cntError) / stats->cntDatagrams);
     _output_outoforder(stats);
     fflush(stdout);
 }
 
-void udp_output_read_enhanced (struct TransferInfo *stats) {
-    HEADING_PRINT_COND(report_bw_jitter_loss_enhanced);
-    _print_stats_common(stats);
-    if (!stats->cntIPG) {
-	printf(report_bw_jitter_loss_suppress_enhanced_format, stats->common->transferIDStr,
-	       stats->ts.iStart, stats->ts.iEnd,
-	       outbuffer, outbufferext,
-	       0.0, stats->cntError,
-	       stats->cntDatagrams,
-	       0.0,0.0,0.0,0.0,0.0,0.0);
-    } else {
-	if ((stats->transit.minTransit > UNREALISTIC_LATENCYMINMAX) ||
-	    (stats->transit.minTransit < UNREALISTIC_LATENCYMINMIN)) {
-	    printf(report_bw_jitter_loss_suppress_enhanced_format, stats->common->transferIDStr,
-		   stats->ts.iStart, stats->ts.iEnd,
-		   outbuffer, outbufferext,
-		   stats->jitter*1000.0, stats->cntError, stats->cntDatagrams,
-		   (100.0 * stats->cntError) / stats->cntDatagrams,
-		   (stats->cntIPG / stats->IPGsum));
-	} else {
-	    double meantransit = (stats->transit.cntTransit > 0) ? (stats->transit.sumTransit / stats->transit.cntTransit) : 0;
-	    set_netpowerbuf(meantransit, stats);
-	    printf(report_bw_jitter_loss_enhanced_format, stats->common->transferIDStr,
-		   stats->ts.iStart, stats->ts.iEnd,
-		   outbuffer, outbufferext,
-		   stats->jitter*1000.0, stats->cntError, stats->cntDatagrams,
-		   (100.0 * stats->cntError) / stats->cntDatagrams,
-		   (meantransit * 1e3),
-		   stats->transit.minTransit*1e3,
-		   stats->transit.maxTransit*1e3,
-		   (stats->transit.cntTransit < 2) ? 0 : sqrt(stats->transit.m2Transit / (stats->transit.cntTransit - 1)) / 1e3,
-		   (stats->cntIPG / stats->IPGsum),
-		   netpower_buf);
-	}
-    }
-    if (stats->latency_histogram) {
-	histogram_print(stats->latency_histogram, stats->ts.iStart, stats->ts.iEnd);
-    }
-    _output_outoforder(stats);
-    fflush(stdout);
-}
-
 void udp_output_read_enhanced_triptime (struct TransferInfo *stats) {
     HEADING_PRINT_COND(report_bw_jitter_loss_enhanced_triptime);
     _print_stats_common(stats);
+
     if (!stats->cntIPG) {
 	printf(report_bw_jitter_loss_suppress_enhanced_format, stats->common->transferIDStr,
 	       stats->ts.iStart, stats->ts.iEnd,
@@ -588,29 +663,30 @@ void udp_output_read_enhanced_triptime (struct TransferInfo *stats) {
 	       stats->cntDatagrams,
 	       0.0,0.0,0.0,0.0,0.0,0.0);
     } else {
-	if ((stats->transit.minTransit > UNREALISTIC_LATENCYMINMAX) ||
-	    (stats->transit.minTransit < UNREALISTIC_LATENCYMINMIN)) {
+	if ((stats->transit.current.min > UNREALISTIC_LATENCYMINMAX) ||
+	    (stats->transit.current.min < UNREALISTIC_LATENCYMINMIN)) {
 	    printf(report_bw_jitter_loss_suppress_enhanced_format, stats->common->transferIDStr,
 		   stats->ts.iStart, stats->ts.iEnd,
 		   outbuffer, outbufferext,
-		   stats->jitter*1000.0, stats->cntError, stats->cntDatagrams,
+		   (stats->jitter * 1e3), stats->cntError, stats->cntDatagrams,
 		   (100.0 * stats->cntError) / stats->cntDatagrams,
 		   (stats->cntIPG / stats->IPGsum));
 	} else {
-	    double meantransit = (stats->transit.cntTransit > 0) ? (stats->transit.sumTransit / stats->transit.cntTransit) : 0;
+	    double meantransit = (stats->transit.current.cnt > 0) ? (stats->transit.current.sum / stats->transit.current.cnt) : 0;
 	    int lambda =  ((stats->IPGsum > 0.0) ? (round (stats->cntIPG / stats->IPGsum)) : 0.0);
-	    double variance = (stats->transit.cntTransit < 2) ? 0 : (sqrt(stats->transit.m2Transit / (stats->transit.cntTransit - 1)) / 1e6);
+	    double variance = (stats->transit.current.cnt < 2) ? 0 : \
+		(sqrt(stats->transit.current.m2 / (stats->transit.current.cnt - 1)));
 	    set_llawbuf_udp(lambda, meantransit, variance, stats);
 	    set_netpowerbuf(meantransit, stats);
 	    printf(report_bw_jitter_loss_enhanced_triptime_format, stats->common->transferIDStr,
 		   stats->ts.iStart, stats->ts.iEnd,
 		   outbuffer, outbufferext,
-		   stats->jitter*1000.0, stats->cntError, stats->cntDatagrams,
+		   (stats->jitter * 1e3), stats->cntError, stats->cntDatagrams,
 		   (100.0 * stats->cntError) / stats->cntDatagrams,
 		   (meantransit * 1e3),
-		   stats->transit.minTransit*1e3,
-		   stats->transit.maxTransit*1e3,
-		   (stats->transit.cntTransit < 2) ? 0 : sqrt(stats->transit.m2Transit / (stats->transit.cntTransit - 1)) / 1e3,
+		   stats->transit.current.min * 1e3,
+		   stats->transit.current.max * 1e3,
+		   (stats->transit.current.cnt < 2) ? 0 : (1e3 * variance), // convert from sec to ms
 		   (stats->cntIPG / stats->IPGsum),
 		   llaw_buf,
 		   netpower_buf);
@@ -637,29 +713,29 @@ void udp_output_read_enhanced_triptime_isoch (struct TransferInfo *stats) {
 	// If the min latency is out of bounds of a realistic value
 	// assume the clocks are not synched and suppress the
 	// latency output
-	if ((stats->transit.minTransit > UNREALISTIC_LATENCYMINMAX) ||
-	    (stats->transit.minTransit < UNREALISTIC_LATENCYMINMIN)) {
+	if ((stats->transit.current.min > UNREALISTIC_LATENCYMINMAX) ||
+	    (stats->transit.current.min < UNREALISTIC_LATENCYMINMIN)) {
 	    printf(report_bw_jitter_loss_suppress_enhanced_format, stats->common->transferIDStr,
 		   stats->ts.iStart, stats->ts.iEnd,
 		   outbuffer, outbufferext,
-		   stats->jitter*1000.0, stats->cntError, stats->cntDatagrams,
+		   (stats->jitter * 1e3), stats->cntError, stats->cntDatagrams,
 		   (100.0 * stats->cntError) / stats->cntDatagrams,
 		   (stats->cntIPG / stats->IPGsum));
 	} else {
-	    double meantransit = (stats->transit.cntTransit > 0) ? (stats->transit.sumTransit / stats->transit.cntTransit) : 0;
+	    double meantransit = (stats->transit.current.cnt > 0) ? (stats->transit.current.sum / stats->transit.current.cnt) : 0;
 	    int lambda =  ((stats->IPGsum > 0.0) ? (round (stats->cntIPG / stats->IPGsum)) : 0.0);
-	    double variance = (stats->transit.cntTransit < 2) ? 0 : (sqrt(stats->transit.m2Transit / (stats->transit.cntTransit - 1)) / 1e6);
+	    double variance = (stats->transit.current.cnt < 2) ? 0 : (sqrt(stats->transit.current.m2 / (stats->transit.current.cnt - 1)));
 	    set_llawbuf_udp(lambda, meantransit, variance, stats);
 	    set_netpowerbuf(meantransit, stats);
 	    printf(report_bw_jitter_loss_enhanced_isoch_format, stats->common->transferIDStr,
 		   stats->ts.iStart, stats->ts.iEnd,
 		   outbuffer, outbufferext,
-		   stats->jitter*1e3, stats->cntError, stats->cntDatagrams,
+		   (stats->jitter * 1e3), stats->cntError, stats->cntDatagrams,
 		   (100.0 * stats->cntError) / stats->cntDatagrams,
 		   (meantransit * 1e3),
-		   stats->transit.minTransit*1e3,
-		   stats->transit.maxTransit*1e3,
-		   (stats->transit.cntTransit < 2) ? 0 : sqrt(stats->transit.m2Transit / (stats->transit.cntTransit - 1)) / 1e3,
+		   stats->transit.current.min * 1e3,
+		   stats->transit.current.max * 1e3,
+		   (stats->transit.current.cnt < 2) ? 0 : 1e3 * (sqrt(stats->transit.current.m2 / (stats->transit.current.cnt - 1))),
 		   (stats->cntIPG / stats->IPGsum),
 		   llaw_buf,
 		   netpower_buf,
@@ -770,7 +846,7 @@ void udp_output_sumcnt_read_enhanced (struct TransferInfo *stats) {
     printf(report_sumcnt_bw_read_enhanced_format, stats->threadcnt,
 	   stats->ts.iStart, stats->ts.iEnd,
 	   outbuffer, outbufferext,
-	   stats->jitter*1000.0, stats->cntError, stats->cntDatagrams,
+	   (stats->jitter * 1e3), stats->cntError, stats->cntDatagrams,
 	   (100.0 * stats->cntError) / stats->cntDatagrams);
     if (stats->cntOutofOrder > 0) {
 	if (isSumOnly(stats->common)) {
@@ -907,7 +983,7 @@ void tcp_output_sum_write_enhanced (struct TransferInfo *stats) {
 	   stats->sock_callstats.write.WriteCnt,
 	   stats->sock_callstats.write.WriteErr
 #if HAVE_TCP_STATS
-	   ,stats->sock_callstats.write.TCPretry
+	   ,stats->sock_callstats.write.tcpstats.retry
 #endif
     );
     fflush(stdout);
@@ -921,7 +997,7 @@ void tcp_output_sumcnt_write_enhanced (struct TransferInfo *stats) {
 	   stats->sock_callstats.write.WriteCnt,
 	   stats->sock_callstats.write.WriteErr
 #if HAVE_TCP_STATS
-	   ,stats->sock_callstats.write.TCPretry
+	   ,stats->sock_callstats.write.tcpstats.retry
 #endif
     );
     fflush(stdout);
@@ -1010,7 +1086,7 @@ void udp_output_basic_csv (struct TransferInfo *stats) {
 	    stats->ts.iEnd,
 	    stats->cntBytes,
 	    speed,
-	    stats->jitter*1000.0,
+	    (stats->jitter * 1e3),
 	    stats->cntError,
 	    stats->cntDatagrams,
 	    (100.0 * stats->cntError) / stats->cntDatagrams, stats->cntOutofOrder );
@@ -1116,6 +1192,13 @@ static void reporter_output_listener_settings (struct ReportSettings *report) {
     if (isOverrideTOS(report->common)) {
 	fprintf(stdout, "Reflected TOS will be set to 0x%x\n", report->common->RTOS);
     }
+    if (isPrintMSS(report->common)) {
+        if (isTCPMSS(report->common)) {
+	    printf(report_mss, report->sockmaxseg);
+	} else {
+	    printf(report_default_mss, report->sockmaxseg);
+	}
+    }
     if (report->common->TOS) {
 	fprintf(stdout, "TOS will be set to 0x%x\n", report->common->TOS);
     }
@@ -1168,8 +1251,8 @@ static void reporter_output_client_settings (struct ReportSettings *report) {
     if ((isEnhanced(report->common) || isNearCongest(report->common)) && !isUDP(report->common)) {
 	byte_snprintf(outbuffer, sizeof(outbuffer), report->common->BufLen, 'B');
 	outbuffer[(sizeof(outbuffer)-1)] = '\0';
-	if (isTcpDrain(report->common)) {
-	    printf("%s: %s (drain-enabled)\n", client_write_size, outbuffer);
+	if (isTcpWriteTimes(report->common)) {
+	    printf("%s: %s (writetimer-enabled)\n", client_write_size, outbuffer);
 	} else {
 	    printf("%s: %s\n", client_write_size, outbuffer);
 	}
@@ -1190,15 +1273,27 @@ static void reporter_output_client_settings (struct ReportSettings *report) {
     }
     if (isBounceBack(report->common)) {
 	char tmpbuf[40];
-	byte_snprintf(tmpbuf, sizeof(tmpbuf), report->common->BurstSize, 'A');
+	byte_snprintf(tmpbuf, sizeof(tmpbuf), report->common->bbsize, 'A');
 	tmpbuf[39]='\0';
-	printf(client_bounceback, tmpbuf);
+	if (isTcpQuickAck(report->common)) {
+	    printf(client_bounceback, tmpbuf, report->common->bbhold);
+	} else {
+	    printf(client_bounceback_noqack, tmpbuf, report->common->bbhold);
+	}
     }
     if (isFQPacing(report->common)) {
 	byte_snprintf(outbuffer, sizeof(outbuffer), report->common->FQPacingRate, 'a');
 	outbuffer[(sizeof(outbuffer)-1)] = '\0';
         printf(client_fq_pacing,outbuffer);
     }
+    if (isPrintMSS(report->common)) {
+        if (isTCPMSS(report->common)) {
+	    printf(report_mss, report->sockmaxseg);
+	} else {
+	    printf(report_default_mss, report->sockmaxseg);
+	}
+    }
+
     if (isCongestionControl(report->common) && report->common->Congestion) {
 	fprintf(stdout, "TCP congestion control set to %s\n", report->common->Congestion);
     }
@@ -1248,7 +1343,7 @@ static void reporter_output_client_settings (struct ReportSettings *report) {
 
 void reporter_connect_printf_tcp_final (struct ConnectionInfo * report) {
     if (report->connect_times.cnt > 1) {
-        double variance = (report->connect_times.cnt < 2) ? 0 : sqrt(report->connect_times.m2 / (report->connect_times.cnt - 1));
+        double variance = (report->connect_times.cnt < 2) ? 0 : 1e3* (sqrt(report->connect_times.m2 / (report->connect_times.cnt - 1)));
         fprintf(stdout, "[ CT] final connect times (min/avg/max/stdev) = %0.3f/%0.3f/%0.3f/%0.3f ms (tot/err) = %d/%d\n", \
 		report->connect_times.min,  \
 	        (report->connect_times.sum / report->connect_times.cnt), \
@@ -1268,16 +1363,8 @@ void reporter_print_connection_report (struct ConnectionInfo *report) {
     struct sockaddr *peer = ((struct sockaddr*)&report->common->peer);
     outbuffer[0]='\0';
     outbufferext[0]='\0';
-    outbufferext2[0]='\0';
     char *b = &outbuffer[0];
-    if (!isUDP(report->common) && (report->common->socket > 0) && (isPrintMSS(report->common) || isEnhanced(report->common)))  {
-	if (isPrintMSS(report->common) && (report->MSS <= 0)) {
-	    printf(report_mss_unsupported, report->MSS);
-	} else if (report->MSS != -1) {
-	    snprintf(b, SNBUFFERSIZE-strlen(b), " (%s%d)", "MSS=", report->MSS);
-	    b += strlen(b);
-	}
-    }
+
 #if HAVE_DECL_TCP_WINDOW_CLAMP
     if (!isUDP(report->common) && isRxClamp(report->common)) {
 	snprintf(b, SNBUFFERSIZE-strlen(b), " (%s%d)", "clamp=", report->common->ClampSize);
@@ -1318,7 +1405,7 @@ void reporter_print_connection_report (struct ConnectionInfo *report) {
 	b += strlen(b);
     }
     if (isBounceBack(report->common)) {
-	snprintf(b, SNBUFFERSIZE-strlen(b), " (bounce-back)");
+	snprintf(b, SNBUFFERSIZE-strlen(b), " (bb len/hold=%d/%d)", report->common->bbsize, report->common->bbhold);
 	b += strlen(b);
     }
     if (isL2LengthCheck(report->common)) {
@@ -1345,10 +1432,12 @@ void reporter_print_connection_report (struct ConnectionInfo *report) {
 	}
 	b += strlen(b);
     } else if (report->common->TOS) {
-	if (isFullDuplex(report->common)) {
+	if (isFullDuplex(report->common) || isBounceBack(report->common)) {
 	    snprintf(b, SNBUFFERSIZE-strlen(b), " (tos rx/tx=0x%x/0x%x)", report->common->TOS, report->common->TOS);
 	} else if (isReverse(report->common)) {
 	    snprintf(b, SNBUFFERSIZE-strlen(b), " (tos tx=0x%x)", report->common->TOS);
+	} else {
+	    snprintf(b, SNBUFFERSIZE-strlen(b), " (tos rx=0x%x)", report->common->TOS);
 	}
 	b += strlen(b);
     }
@@ -1358,20 +1447,28 @@ void reporter_print_connection_report (struct ConnectionInfo *report) {
 	    b += strlen(b);
 	}
     }
+#if HAVE_DECL_TCP_QUICKACK
+    if (isTcpQuickAck(report->common)) {
+	snprintf(b, SNBUFFERSIZE-strlen(b), " (qack)");
+	b += strlen(b);
+    }
+#endif
+#if HAVE_TCP_STATS
+    if (!isUDP(report->common) && (report->tcpinitstats.isValid)) {
+	snprintf(b, SNBUFFERSIZE-strlen(b), " (icwnd/mss/irtt=%u/%u/%u)", \
+		 report->tcpinitstats.cwnd, report->tcpinitstats.mss_negotiated, report->tcpinitstats.rtt);
+	b += strlen(b);
+    }
+#endif
+
     if (!isServerReverse(report->common) && (isEnhanced(report->common) || isConnectOnly(report->common))) {
 	if (report->connect_timestamp.tv_sec > 0) {
 	    struct tm ts;
 	    ts = *localtime(&report->connect_timestamp.tv_sec);
 	    char now_timebuf[80];
 	    strftime(now_timebuf, sizeof(now_timebuf), "%Y-%m-%d %H:%M:%S (%Z)", &ts);
-	    if (!isUDP(report->common) && (report->common->ThreadMode == kMode_Client)) {
-#if HAVE_TCP_STATS
-		if (report->init_cond.connecttime > 0.0) {
-		    snprintf(b, SNBUFFERSIZE-strlen(b), " (irtt/icwnd=%u/%u)", report->init_cond.rtt, report->init_cond.cwnd);
-		    b += strlen(b);
-		}
-#endif
-		snprintf(b, SNBUFFERSIZE-strlen(b), " (ct=%4.2f ms) on %s", report->init_cond.connecttime, now_timebuf);
+	    if (!isUDP(report->common) && (report->common->ThreadMode == kMode_Client) && (report->tcpinitstats.connecttime > 0)) {
+		snprintf(b, SNBUFFERSIZE-strlen(b), " (ct=%4.2f ms) on %s", report->tcpinitstats.connecttime, now_timebuf);
 	    } else {
 		snprintf(b, SNBUFFERSIZE-strlen(b), " on %s", now_timebuf);
 	    }
@@ -1539,7 +1636,7 @@ void reporter_print_server_relay_report (struct ServerRelay *report) {
     if (!isEnhanced(report->info.common)) {
 	udp_output_read(&report->info);
     } else {
-	udp_output_read_enhanced(&report->info);
+	udp_output_read_enhanced_triptime(&report->info);
     }
     fflush(stdout);
 }
