@@ -71,6 +71,7 @@
 #include "isochronous.hpp"
 #include "pdfs.h"
 #include "payloads.h"
+#include "PerfSocket.hpp"
 #include <math.h>
 
 static int reversetest = 0;
@@ -107,8 +108,14 @@ static int tapif = 0;
 static int tunif = 0;
 static int hideips = 0;
 static int bounceback = 0;
-static int tcpdrain;
-static int overridetos;
+static int bouncebackhold = 0;
+static int bouncebackperiod = 0;
+static int overridetos = 0;
+static int notcpbbquickack = 0;
+static int tcpquickack = 0;
+static int notcpbbquickack_cliset = 0;
+static int congest = 0;
+static int tcpwritetimes = 0;
 
 void Settings_Interpret(char option, const char *optarg, struct thread_Settings *mExtSettings);
 // apply compound settings after the command line has been fully parsed
@@ -127,7 +134,7 @@ static void generate_permit_key(struct thread_Settings *mExtSettings);
 const struct option long_options[] =
 {
 {"singleclient",     no_argument, NULL, '1'},
-#if 0 // to be released in 2.1.6
+#if 0
 {"v4",               no_argument, NULL, '4'},
 {"v6",               no_argument, NULL, '6'},
 #endif
@@ -156,9 +163,11 @@ const struct option long_options[] =
 // more esoteric options
 {"awdl",             no_argument, NULL, 'A'},
 {"bind",       required_argument, NULL, 'B'},
-#if 0 // to be released in 2.1.6
-{"bounce-back", optional_argument, &bounceback, 1},
-#endif
+{"bounceback", no_argument, &bounceback, 1},
+{"bounceback-congest", no_argument, &congest, 1},
+{"bounceback-hold", required_argument, &bouncebackhold, 1},
+{"bounceback-no-quickack", no_argument, &notcpbbquickack, 1},
+{"bounceback-period", required_argument, &bouncebackperiod, 1},
 {"compatibility",    no_argument, NULL, 'C'},
 {"daemon",           no_argument, NULL, 'D'},
 {"file_input", required_argument, NULL, 'F'},
@@ -207,10 +216,11 @@ const struct option long_options[] =
 {"permit-key-timeout", required_argument, &permitkeytimeout, 1},
 {"burst-size", required_argument, &burstsize, 1},
 {"burst-period", required_argument, &burstperiodic, 1},
-{"tcp-drain", no_argument, &tcpdrain, 1},
 {"tos-override", required_argument, &overridetos, 1},
 {"tcp-rx-window-clamp", required_argument, &rxwinclamp, 1},
+{"tcp-quickack", no_argument, &tcpquickack, 1},
 {"tcp-write-prefetch", required_argument, &txnotsentlowwater, 1}, // see doc/DESIGN_NOTES
+{"tcp-write-times", no_argument, &tcpwritetimes, 1},
 {"tap-dev", optional_argument, &tapif, 1},
 {"tun-dev", optional_argument, &tunif, 1},
 {"NUM_REPORT_STRUCTS", required_argument, &numreportstructs, 1},
@@ -275,12 +285,9 @@ const char short_options[] = "146b:c:def:hi:l:mn:o:p:rst:uvw:x:y:zAB:CDF:H:IL:M:
 #define DEFAULTS()
 
 const long kDefault_UDPRate = 1024 * 1024; // -u  if set, 1 Mbit/sec
-const int  kDefault_UDPBufLen = 1470;      // -u  if set, read/write 1470 bytes
-
-// v4: 1470 bytes UDP payload will fill one and only one ethernet datagram (IPv4 overhead is 20 bytes)
-const int  kDefault_UDPBufLenV6 = 1450;      // -u  if set, read/write 1470 bytes
-// v6: 1450 bytes UDP payload will fill one and only one ethernet datagram (IPv6 overhead is 40 bytes)
 const int kDefault_TCPBufLen = 128 * 1024; // TCP default read/write size
+const int kDefault_BBTCPBufLen = 100; // default bounce-back size in bytes
+
 
 /* -------------------------------------------------------------------
  * Initialize all settings to defaults.
@@ -429,6 +436,8 @@ void Settings_Copy (struct thread_Settings *from, struct thread_Settings **into,
     mbuflen += TAPBYTESSLOP;
 #endif
     (*into)->mBuf = new char[mbuflen];
+    memset((*into)->mBuf, 0, mbuflen);
+
 #ifdef HAVE_THREAD_DEBUG
     thread_debug("Copy Settings: MBUF malloc %d bytes (%p)", mbuflen, (void *) (*into)->mBuf);
 #endif
@@ -623,7 +632,9 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
             break;
 
         case 'm': // print TCP MSS
+#if HAVE_DECL_TCP_MAXSEG
             setPrintMSS(mExtSettings);
+#endif
             break;
 
         case 'n': // bytes of data
@@ -815,9 +826,11 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
             break;
 
         case 'M': // specify TCP MSS (maximum segment size)
-            mExtSettings->mMSS = byte_atoi(optarg);
-            setPrintMSS(mExtSettings);
+#if HAVE_DECL_TCP_MAXSEG
+	    mExtSettings->mMSS = byte_atoi(optarg);
             setTCPMSS(mExtSettings);
+            setPrintMSS(mExtSettings);
+#endif
             break;
 
         case 'N': // specify TCP nodelay option (disable Jacobson's Algorithm)
@@ -1083,26 +1096,30 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
 		fprintf(stderr, "--tcp-rx-window-clamp not supported on this platform\n");
 #endif
 	    }
-	    if (tcpdrain) {
-		tcpdrain = 0;
-#if HAVE_DECL_TCP_NOTSENT_LOWAT
-		setTcpDrain(mExtSettings);
-		setEnhanced(mExtSettings);
-#else
-		fprintf(stderr, "--tcp-drain not supported on this platform\n");
+	    if (tcpwritetimes) {
+		tcpwritetimes = 0;
+		setTcpWriteTimes(mExtSettings);
+	    }
+	    if (notcpbbquickack) {
+		notcpbbquickack = 0;
+		notcpbbquickack_cliset = 1;
+	    }
+	    if (tcpquickack) {
+		tcpquickack = 0;
+#if HAVE_DECL_TCP_QUICKACK
+		setTcpQuickAck(mExtSettings);
 #endif
 	    }
+	    if (congest) {
+		congest= 0;
+		setCongest(mExtSettings);
+	    }
 	    if (txnotsentlowwater) {
 		txnotsentlowwater = 0;
 #if HAVE_DECL_TCP_NOTSENT_LOWAT
 		mExtSettings->mWritePrefetch = byte_atoi(optarg);
 		setWritePrefetch(mExtSettings);
 		setEnhanced(mExtSettings);
-		mExtSettings->mHistBins = 100000; // 10 seconds wide
-		mExtSettings->mHistBinsize = 100; // 100 usec bins
-		mExtSettings->mHistUnits = 6;  // usecs 10 pow(x)
-		mExtSettings->mHistci_lower = 5;
-		mExtSettings->mHistci_upper = 95;
 #else
 		fprintf(stderr, "--tcp-write-prefetch not supported on this platform\n");
 #endif
@@ -1110,12 +1127,17 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
 	    if (burstperiodic) {
 		burstperiodic = 0;
 		setPeriodicBurst(mExtSettings);
-		if (optarg) {
+		if (optarg && (atof(optarg) > 1e-5)) { // limit to 10 usecs
 		    mExtSettings->mFPS = 1.0/atof(optarg);
+		} else {
+		    if (atof(optarg) != 0)
+			fprintf(stderr, "WARN: burst-period too small, must be greater than 10 usecs\n");
+		    unsetPeriodicBurst(mExtSettings);
 		}
 	    }
 	    if (burstsize) {
 		burstsize = 0;
+		setPeriodicBurst(mExtSettings);
 		if (optarg) {
 		    mExtSettings->mBurstSize = byte_atoi(optarg);
 		}
@@ -1159,6 +1181,27 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
 	    if (bounceback) {
 		bounceback = 0;
 		setBounceBack(mExtSettings);
+		setNoDelay(mExtSettings);
+		setEnhanced(mExtSettings);
+	    }
+	    if (bouncebackhold) {
+		bouncebackhold = 0;
+		if (optarg)
+		    //cli units is ms, working units is us
+		    mExtSettings->mBounceBackHold = int(atof(optarg) * 1e3);
+		else
+		    mExtSettings->mBounceBackHold = 0;
+	    }
+	    if (bouncebackperiod) {
+		bouncebackperiod = 0;
+		setPeriodicBurst(mExtSettings);
+		if (optarg && (atof(optarg) > 1e-2)) { // limit to 10 usecs
+		    mExtSettings->mFPS = 1e3/atof(optarg); // cli units is ms
+		} else {
+		    if (atof(optarg) != 0)
+			fprintf(stderr, "WARN: bouncback-period too small, must be greater than 10 usecs\n");
+		    unsetPeriodicBurst(mExtSettings);
+		}
 	    }
 	    break;
         default: // ignore unknown
@@ -1241,7 +1284,10 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
 		mExtSettings->mBufLen = kDefault_UDPBufLen;
 	    }
 	} else {
-	    mExtSettings->mBufLen = kDefault_TCPBufLen;
+	    if (isBounceBack(mExtSettings))
+	        mExtSettings->mBufLen = kDefault_BBTCPBufLen;
+	    else
+	        mExtSettings->mBufLen = kDefault_TCPBufLen;
 	}
     }
     if (!mExtSettings->mPortLast)
@@ -1382,10 +1428,25 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
 		bail = true;
 	    }
 	}
-	if (isBounceBack(mExtSettings) && (static_cast<int> (mExtSettings->mBurstSize) < mExtSettings->mBufLen)) {
-	    fprintf(stderr, "WARN: options of --burst-size for bounce-back is being set to -l length of %d\n", mExtSettings->mBufLen);
+	if (isBounceBack(mExtSettings)) {
+	    if (static_cast<int> (mExtSettings->mBurstSize) > 0) {
+		fprintf(stderr, "WARN: options of --burst-size for bounce-back ignored, use -l sets size\n");
+	    }
+	    mExtSettings->mBounceBackBytes = mExtSettings->mBufLen;
 	    mExtSettings->mBurstSize = mExtSettings->mBufLen;
+#if HAVE_DECL_TCP_QUICKACK
+	    if (notcpbbquickack_cliset && isTcpQuickAck(mExtSettings)) {
+		fprintf(stderr, "ERROR: --tcp-quickack and --bounceback-no-quickack are mutually exclusive\n");
+		bail = true;
+	    }
+	    // be wary of double negatives here
+	    if (!notcpbbquickack_cliset && (mExtSettings->mBounceBackHold > 0)) {
+		setTcpQuickAck(mExtSettings);
+	    }
+#endif
+
 	}
+
 	if (isPeriodicBurst(mExtSettings)) {
 	    if (isIsochronous(mExtSettings)) {
 		fprintf(stderr, "ERROR: options of --burst-period and --isochronous cannot be applied together\n");
@@ -1395,12 +1456,9 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
 		bail = true;
 	    } else if (static_cast<int> (mExtSettings->mBurstSize) == 0) {
 	        mExtSettings->mBurstSize = byte_atoi("1M"); //default to 1 Mbyte
-	    } else if (isBounceBack(mExtSettings)) {
-		fprintf(stderr, "ERROR: options of --burst-period and --bounce-back cannot be applied together\n");
-		bail = true;
 	    }
 	    if (static_cast<int> (mExtSettings->mBurstSize) < mExtSettings->mBufLen) {
-		fprintf(stderr, "ERROR: option of --burst-size must be equal or larger to write length (-l)\n");
+		fprintf(stderr, "ERROR: option of --burst-size %d must be equal or larger to write length (-l) %d\n", mExtSettings->mBurstSize, mExtSettings->mBufLen);
 		bail = true;
 	    }
 	} else if (!isBounceBack(mExtSettings) && (static_cast<int> (mExtSettings->mBurstSize) > 0)) {
@@ -1437,9 +1495,9 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
 		fprintf(stderr, "WARN: setting of option --tcp-write-prefetch is not supported with -u UDP\n");
 		unsetWritePrefetch(mExtSettings);
 	    }
-	    if (isTcpDrain(mExtSettings)) {
-		fprintf(stderr, "WARN: setting of option --tcp-drain is not supported with -u UDP\n");
-		unsetTcpDrain(mExtSettings);
+	    if (isTcpQuickAck(mExtSettings)) {
+		fprintf(stderr, "WARN: setting of option --tcp-quickack is not supported with -u UDP\n");
+		unsetWritePrefetch(mExtSettings);
 	    }
 
 	    {
@@ -1499,8 +1557,13 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
 		bail = true;
 	    }
 	}
-	if (!isReverse(mExtSettings) && !isFullDuplex(mExtSettings) && isHistogram(mExtSettings) && !isWritePrefetch(mExtSettings)) {
-	    fprintf(stderr, "WARN: option of --histograms on the client requires --tcp-write-prefetch\n");
+	if (!isReverse(mExtSettings) && !isFullDuplex(mExtSettings) && isHistogram(mExtSettings)){
+	    setTcpWriteTimes(mExtSettings);
+	    mExtSettings->mHistBins = 100000; // 10 seconds wide
+	    mExtSettings->mHistBinsize = 100; // 100 usec bins
+	    mExtSettings->mHistUnits = 6;  // usecs 10 pow(x)
+	    mExtSettings->mHistci_lower = 5;
+	    mExtSettings->mHistci_upper = 95;
 	}
 	if (isCongestionControl(mExtSettings) && isReverse(mExtSettings)) {
 	    fprintf(stderr, "ERROR: tcp congestion control -Z and --reverse cannot be applied together\n");
@@ -1547,10 +1610,6 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
             fprintf(stderr, "WARN: setting of option --tcp-write-prefetch is not supported on the server\n");
 	    unsetWritePrefetch(mExtSettings);
 	}
-        if (isTcpDrain(mExtSettings)) {
-            fprintf(stderr, "WARN: setting of option --tcp-drain is not supported on the server\n");
-	    unsetTcpDrain(mExtSettings);
-	}
         if (isIncrSrcIP(mExtSettings)) {
             fprintf(stderr, "WARN: setting of option --incr-srcip is not supported on the server\n");
 	}
@@ -1910,8 +1969,14 @@ void Settings_ReadClientSettingsV1 (struct thread_Settings **client, struct clie
     (*client)->mTID = thread_zeroid();
     (*client)->mPort = static_cast<unsigned short>(ntohl(hdr->mPort));
     (*client)->mThreads = 1;
+    struct thread_Settings *tmpSettings  = *client;
     if (hdr->mBufLen != 0) {
 	(*client)->mBufLen = ntohl(hdr->mBufLen);
+	setBuflenSet(tmpSettings);
+    } else {
+#ifdef DEFAULT_PAYLOAD_LEN_PER_MTU_DISCOVERY
+	checksock_max_udp_payload(tmpSettings);
+#endif
     }
     (*client)->mAmount = ntohl(hdr->mAmount);
     if (((*client)->mAmount & 0x80000000) > 0) {
@@ -2117,7 +2182,10 @@ int Settings_GenerateClientHdr (struct thread_Settings *client, void *testhdr, s
 
     // Check the corner case of small packets and trip times
     if (isUDP(client) && isSmallTripTime(client)) {
+        int buflen = (client->mBufLen < (int) sizeof(struct client_udpsmall_testhdr)) ? client->mBufLen \
+	             : sizeof(struct client_udpsmall_testhdr);
 	struct client_udpsmall_testhdr *hdr = static_cast<struct client_udpsmall_testhdr *>(testhdr);
+	memset(hdr, 0, buflen);
 	hdr->flags = htons(HEADER16_SMALL_TRIPTIMES);
 #ifdef HAVE_THREAD_DEBUG
 	thread_debug("UDP small trip times flags = %X", ntohs(hdr->flags));
@@ -2138,7 +2206,9 @@ int Settings_GenerateClientHdr (struct thread_Settings *client, void *testhdr, s
     // Now setup UDP and TCP specific passed settings from client to server
     if (isUDP(client)) { // UDP test information passed in every packet per being stateless
 	struct client_udp_testhdr *hdr = static_cast<struct client_udp_testhdr *>(testhdr);
-	memset(hdr, 0, sizeof(struct client_udp_testhdr));
+        int buflen = (client->mBufLen < (int) sizeof(struct client_udp_testhdr)) ? client->mBufLen \
+	             : sizeof(struct client_udp_testhdr);
+	memset(hdr, 0, buflen);
 	flags |= HEADER_SEQNO64B; // use 64 bit by default
 	flags |= HEADER_EXTEND;
 	hdr->extend.version_u = htonl(IPERF_VERSION_MAJORHEX);
@@ -2242,9 +2312,11 @@ int Settings_GenerateClientHdr (struct thread_Settings *client, void *testhdr, s
 #endif
 	if (isBounceBack(client)) {
 	    flags = HEADER_BOUNCEBACK;
-	    len = sizeof(struct bounce_back_datagram_hdr);
+	    len = sizeof(struct bounceback_hdr);
 	} else {
-	    memset(hdr, 0, sizeof(struct client_tcp_testhdr));
+	    int buflen = (client->mBufLen < (int) sizeof(struct client_tcp_testhdr)) ? client->mBufLen \
+	                 : sizeof(struct client_tcp_testhdr);
+	    memset(hdr, 0, buflen);
 	    flags |= HEADER_EXTEND;
 	    hdr->extend.version_u = htonl(IPERF_VERSION_MAJORHEX);
 	    hdr->extend.version_l = htonl(IPERF_VERSION_MINORHEX);
@@ -2286,6 +2358,11 @@ int Settings_GenerateClientHdr (struct thread_Settings *client, void *testhdr, s
 		upperflags  |= HEADER_WRITEPREFETCH;
 		hdr->extend.TCPWritePrefetch = htonl((long)client->mWritePrefetch);
 	    }
+#endif
+#if HAVE_DECL_TCP_QUICKACK
+	    if (isTcpQuickAck(client) && (!isReverse(client) || isFullDuplex(client))) {
+		upperflags  |= HEADER_TCPQUICKACK;
+	    }
 #endif
 	    if (isIsochronous(client) || isPeriodicBurst(client)) {
 		if (isPeriodicBurst(client)) {
